@@ -23,7 +23,7 @@ __copyright__ = '(C) 2013, Alexander Bruy'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '564579199220ae06abd248f0dc80b7ff57b80128'
+__revision__ = '93e761dd2396af21db9bb93e26d7cba680c906ca'
 
 import os
 
@@ -57,6 +57,7 @@ class roughness(GdalAlgorithm):
         self.addParameter(QgsProcessingParameterRasterLayer(self.INPUT, self.tr('Input layer')))
         self.addParameter(QgsProcessingParameterBand(self.BAND,
                                                      self.tr('Band number'),
+                                                     1,
                                                      parentLayerParameterName=self.INPUT))
         self.addParameter(QgsProcessingParameterBoolean(self.COMPUTE_EDGES,
                                                         self.tr('Compute edges'),
@@ -98,6 +99,7 @@ class roughness(GdalAlgorithm):
         arguments.append(inLayer.source())
 
         out = self.parameterAsOutputLayer(parameters, self.OUTPUT, context)
+        self.setOutputValue(self.OUTPUT, out)
         arguments.append(out)
 
         arguments.append('-of')
