@@ -23,7 +23,7 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '564579199220ae06abd248f0dc80b7ff57b80128'
+__revision__ = '93e761dd2396af21db9bb93e26d7cba680c906ca'
 
 import codecs
 import sys
@@ -46,7 +46,8 @@ from qgis.PyQt.QtCore import (
     pyqtSignal,
     QDataStream,
     QIODevice,
-    QUrl)
+    QUrl,
+    QTimer)
 from qgis.PyQt.QtWidgets import (QGraphicsView,
                                  QTreeWidget,
                                  QMessageBox,
@@ -61,7 +62,9 @@ from qgis.PyQt.QtWidgets import (QGraphicsView,
                                  QVBoxLayout,
                                  QGridLayout,
                                  QFrame,
-                                 QLineEdit)
+                                 QLineEdit,
+                                 QToolButton,
+                                 QAction)
 from qgis.PyQt.QtGui import (QIcon,
                              QImage,
                              QPainter,
@@ -70,7 +73,7 @@ from qgis.PyQt.QtSvg import QSvgGenerator
 from qgis.PyQt.QtPrintSupport import QPrinter
 from qgis.core import (Qgis,
                        QgsApplication,
-                       QgsProcessingAlgorithm,
+                       QgsProcessing,
                        QgsProject,
                        QgsSettings,
                        QgsMessageLog,
@@ -92,6 +95,8 @@ from processing.modeler.ModelerParametersDialog import ModelerParametersDialog
 from processing.modeler.ModelerUtils import ModelerUtils
 from processing.modeler.ModelerScene import ModelerScene
 from processing.modeler.ProjectProvider import PROJECT_PROVIDER_ID
+from processing.script.ScriptEditorDialog import ScriptEditorDialog
+from processing.core.ProcessingConfig import ProcessingConfig
 from qgis.utils import iface
 
 
@@ -253,6 +258,13 @@ class ModelerDialog(BASE, WIDGET):
             self.mToolbar.setIconSize(iface.iconSize())
             self.setStyleSheet(iface.mainWindow().styleSheet())
 
+        self.toolbutton_export_to_script = QToolButton()
+        self.toolbutton_export_to_script.setPopupMode(QToolButton.InstantPopup)
+        self.export_to_script_algorithm_action = QAction(QCoreApplication.translate('ModelerDialog', 'Export as Script Algorithm…'))
+        self.toolbutton_export_to_script.addActions([self.export_to_script_algorithm_action])
+        self.mToolbar.insertWidget(self.mActionExportImage, self.toolbutton_export_to_script)
+        self.export_to_script_algorithm_action.triggered.connect(self.export_as_script_algorithm)
+
         self.mActionOpen.setIcon(
             QgsApplication.getThemeIcon('/mActionFileOpen.svg'))
         self.mActionSave.setIcon(
@@ -275,8 +287,8 @@ class ModelerDialog(BASE, WIDGET):
             QgsApplication.getThemeIcon('/mActionSaveAsPDF.svg'))
         self.mActionExportSvg.setIcon(
             QgsApplication.getThemeIcon('/mActionSaveAsSVG.svg'))
-        #self.mActionExportPython.setIcon(
-        #    QgsApplication.getThemeIcon('/mActionSaveAsPython.svg'))
+        self.toolbutton_export_to_script.setIcon(
+            QgsApplication.getThemeIcon('/mActionSaveAsPython.svg'))
         self.mActionEditHelp.setIcon(
             QgsApplication.getThemeIcon('/mActionEditHelpContent.svg'))
         self.mActionRun.setIcon(
@@ -288,8 +300,6 @@ class ModelerDialog(BASE, WIDGET):
         self.tabifyDockWidget(self.inputsDock, self.algorithmsDock)
         self.inputsDock.raise_()
 
-        self.zoom = 1
-
         self.setWindowFlags(Qt.WindowMinimizeButtonHint |
                             Qt.WindowMaximizeButtonHint |
                             Qt.WindowCloseButtonHint)
@@ -304,6 +314,7 @@ class ModelerDialog(BASE, WIDGET):
         self.view.setScene(self.scene)
         self.view.setAcceptDrops(True)
         self.view.ensureVisible(0, 0, 10, 10)
+        self.view.scale(QgsApplication.desktop().logicalDpiX() / 96, QgsApplication.desktop().logicalDpiX() / 96)
 
         def _dragEnterEvent(event):
             if event.mimeData().hasText() or event.mimeData().hasFormat('application/x-vnd.qgis.qgis.algorithmid'):
@@ -312,19 +323,26 @@ class ModelerDialog(BASE, WIDGET):
                 event.ignore()
 
         def _dropEvent(event):
+            def alg_dropped(algorithm_id, pos):
+                alg = QgsApplication.processingRegistry().createAlgorithmById(algorithm_id)
+                if alg is not None:
+                    self._addAlgorithm(alg, pos)
+                else:
+                    assert False, algorithm_id
+
+            def input_dropped(id, pos):
+                if id in [param.id() for param in QgsApplication.instance().processingRegistry().parameterTypes()]:
+                    self.addInputOfType(itemId, pos)
+
             if event.mimeData().hasFormat('application/x-vnd.qgis.qgis.algorithmid'):
                 data = event.mimeData().data('application/x-vnd.qgis.qgis.algorithmid')
                 stream = QDataStream(data, QIODevice.ReadOnly)
                 algorithm_id = stream.readQString()
-                alg = QgsApplication.processingRegistry().createAlgorithmById(algorithm_id)
-                if alg is not None:
-                    self._addAlgorithm(alg, event.pos())
-                else:
-                    assert False, algorithm_id
+                QTimer.singleShot(0, lambda id=algorithm_id, pos=self.view.mapToScene(event.pos()): alg_dropped(id, pos))
+                event.accept()
             elif event.mimeData().hasText():
                 itemId = event.mimeData().text()
-                if itemId in [param.id() for param in QgsApplication.instance().processingRegistry().parameterTypes()]:
-                    self.addInputOfType(itemId, event.pos())
+                QTimer.singleShot(0, lambda id=itemId, pos=self.view.mapToScene(event.pos()): input_dropped(id, pos))
                 event.accept()
             else:
                 event.ignore()
@@ -402,7 +420,10 @@ class ModelerDialog(BASE, WIDGET):
         self.algorithmTree.setDragDropMode(QTreeWidget.DragOnly)
         self.algorithmTree.setDropIndicatorShown(True)
 
-        self.algorithmTree.setFilters(QgsProcessingToolboxProxyModel.FilterModeler)
+        filters = QgsProcessingToolboxProxyModel.Filters(QgsProcessingToolboxProxyModel.FilterModeler)
+        if ProcessingConfig.getSetting(ProcessingConfig.SHOW_ALGORITHMS_KNOWN_ISSUES):
+            filters |= QgsProcessingToolboxProxyModel.FilterShowKnownIssues
+        self.algorithmTree.setFilters(filters)
 
         if hasattr(self.searchBox, 'setPlaceholderText'):
             self.searchBox.setPlaceholderText(QCoreApplication.translate('ModelerDialog', 'Search…'))
@@ -487,7 +508,7 @@ class ModelerDialog(BASE, WIDGET):
             self.bar.pushMessage("", self.tr("Model doesn't contain any algorithm and/or parameter and can't be executed"), level=Qgis.Warning, duration=5)
             return
 
-        dlg = AlgorithmDialog(self.model, parent=iface.mainWindow())
+        dlg = AlgorithmDialog(self.model.create(), parent=iface.mainWindow())
         dlg.exec_()
 
     def save(self):
@@ -539,6 +560,7 @@ class ModelerDialog(BASE, WIDGET):
     def zoomActual(self):
         point = self.view.mapToScene(QPoint(self.view.viewport().width() / 2, self.view.viewport().height() / 2))
         self.view.resetTransform()
+        self.view.scale(QgsApplication.desktop().logicalDpiX() / 96, QgsApplication.desktop().logicalDpiX() / 96)
         self.view.centerOn(point)
 
     def zoomToItems(self):
@@ -814,3 +836,9 @@ class ModelerDialog(BASE, WIDGET):
             newX = MARGIN + BOX_WIDTH / 2
             newY = MARGIN * 2 + BOX_HEIGHT + BOX_HEIGHT / 2
         return QPointF(newX, newY)
+
+    def export_as_script_algorithm(self):
+        dlg = ScriptEditorDialog(None)
+
+        dlg.editor.setText('\n'.join(self.model.asPythonCode(QgsProcessing.PythonQgsProcessingAlgorithmSubclass, 4)))
+        dlg.show()
